import sys
import traceback

import myhdl
from myhdl import *

class Error(Exception):
    pass

def fifo(dout, din, re, we, empty, full, clk, maxFilling=sys.maxint):
    
    """ Synchronous fifo model based on a list.
    
    Ports:
    dout -- data out
    din -- data in
    re -- read enable
    we -- write enable
    empty -- empty indication flag
    full -- full indication flag
    clk -- clock input

    Optional parameter:
    maxFilling -- maximum fifo filling, "infinite" by default

    """
    
    memory = []

    @always(clk.posedge)
    def access():
        if we:
            memory.insert(0, din.val)
        if re:
            dout.next = memory.pop()
        filling = len(memory)
        empty.next = (filling == 0)
        full.next = (filling == maxFilling)

    return access

        
def fifo2(dout, din, re, we, empty, full, clk, maxFilling=sys.maxint):
    
    """ Synchronous fifo model based on a list.

    Ports:
    dout -- data out
    din -- data in
    re -- read enable
    we -- write enable
    empty -- empty indication flag
    full -- full indication flag
    clk -- clock input
    
    Optional parameter:
    maxFilling -- maximum fifo filling, "infinite" by default

    """
    
    memory = []

    @always(clk.posedge)
    def access():
        if we:
            memory.insert(0, din.val)
        if re:
            try:
                dout.next = memory.pop()
            except IndexError:
                raise Exception("Underflow -- Read from empty fifo")
        filling = len(memory)
        empty.next = (filling == 0)
        full.next = (filling == maxFilling)
        if filling > maxFilling:
            raise Exception("Overflow -- Max filling %s exceeded" % maxFilling)

    return access


dout, din, re, we, empty, full, clk = args = [Signal(0) for i in range(7)]

dut = fifo2(dout, din, re, we, empty, full, clk, maxFilling=3)

def clkGen():
    while 1:
        yield delay(10)
        clk.next = not clk

def read():
    yield clk.negedge
    re.next = 1
    yield clk.posedge
    yield delay(1)
    re.next = 0

def write(data):
    yield clk.negedge
    din.next = data
    we.next = 1
    yield clk.posedge
    yield delay(1)
    we.next = 0

def report():
    print("dout: %s empty: %s full: %s" % (hex(dout), empty, full))
    
def test():
    yield write(0x55)
    report()
    yield write(0x77)
    report()
    yield write(0x11)
    report()
    yield join(write(0x22), read())
    report()
    yield join(write(0x33), read())
    report()
    yield read()
    report()
    yield read()
    report()
    yield read()
    report()
    yield read()
    report()
    yield read()
    raise StopSimulation

    
sim = Simulation(clkGen(), test(), dut)
    
def main():
    try:
        sim.run()
    except:
        traceback.print_exc()
    
if __name__ == '__main__':
    main()
           
 



import os

from myhdl import Cosimulation

cmd = 'vsim -c -quiet -pli ../myhdl_vpi.so -do cosim.do dut_inc'

def inc(count, enable, clock, reset, n):
    os.system('vlog -quiet +define+n=%s ../../test/verilog/inc.v' % (n))
    os.system('vlog -quiet +define+n=%s ../../test/verilog/dut_inc.v' % (n)) 

    return Cosimulation(cmd, **locals())
               



from myhdl import Signal, block, delay, instance


class MyTest1:

    def __init__(self):
        self.clock = Signal(bool(0))

    @block
    def test(self):

        @instance
        def func():
            i = 0
            while i <= 100:
                yield delay(10)
                self.clock.next = not self.clock
                i = i + 1

        return func


class MyTest2:

    def __init__(self):
        self.clock = Signal(bool(1))

    @block
    def test(self):

        @instance
        def func():
            i = 0
            while i <= 100:
                yield delay(10)
                self.clock.next = not self.clock
                i = i + 1

        return func


@block
def mytest_bench():
    inst1 = MyTest1()
    inst2 = MyTest2()

    # Two instances are created
    ins1 = inst1.test()
    ins2 = inst2.test()

    return ins1, ins2


# @pytest.mark.xfail
def test_issue_169():
    test_inst = mytest_bench()
    test_inst.verify_convert()




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this librardy; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for the @always decorator """
from random import randrange

from myhdl import (AlwaysError, Signal, Simulation, StopSimulation, delay,
                   intbv, always)
from myhdl._always import _error
from myhdl._Waiter import (_DelayWaiter, _EdgeTupleWaiter, _EdgeWaiter,
                           _SignalTupleWaiter, _SignalWaiter, _Waiter)
from helpers import raises_kind

# random.seed(3) # random, but deterministic

QUIET = 1


def g():
    pass


x = Signal(0)


class TestAlwaysCompilation:

    def testArgIsFunction(self):
        h = 5
        with raises_kind(AlwaysError, _error.ArgType):
            always(delay(3))(h)

    def testArgIsNormalFunction(self):
        with raises_kind(AlwaysError, _error.ArgType):

            @always(delay(3))
            def h():
                yield None

    def testArgHasNoArgs(self):
        with raises_kind(AlwaysError, _error.NrOfArgs):

            @always(delay(3))
            def h(n):
                return n

    def testDecArgType1(self):
        with raises_kind(AlwaysError, _error.DecArgType):

            @always
            def h(n):
                return n

    def testDecArgType2(self):
        with raises_kind(AlwaysError, _error.DecArgType):

            @always(g)
            def h(n):
                return n


def SignalFunc1(a, b, c, d, r):

    @always(a)
    def comb():
        r.next = a + b + c + d

    return comb


def SignalTupleFunc1(a, b, c, d, r):

    @always(a, b, c)
    def comb():
        r.next = a + b + c + d

    return comb


def DelayFunc(a, b, c, d, r):

    @always(delay(3))
    def comb():
        r.next = a + b + c + d

    return comb


def EdgeFunc1(a, b, c, d, r):

    @always(c.posedge)
    def comb():
        r.next = a + b + c + d

    return comb


def EdgeTupleFunc1(a, b, c, d, r):

    @always(c.posedge, d.negedge)
    def comb():
        r.next = a + b + c + d

    return comb


def GeneralFunc(a, b, c, d, r):

    @always(c.posedge, d)
    def comb():
        r.next = a + b + c + d

    return comb


class TestInferWaiter:

    def bench(self, MyHDLFunc, waiterType):

        a, b, c, d, r, s = [Signal(intbv(0)) for __ in range(6)]

        inst_r = MyHDLFunc(a, b, c, d, r)
        assert type(inst_r.waiter) == waiterType

        inst_s = MyHDLFunc(a, b, c, d, s)

        def stimulus():
            for dummy in range(1000):
                yield delay(randrange(1, 10))
                if randrange(2):
                    a.next = randrange(32)
                if randrange(2):
                    b.next = randrange(32)
                c.next = randrange(2)
                d.next = randrange(2)
            raise StopSimulation

        def check():
            while 1:
                yield a, b, c, r, s
                assert r == s

        return inst_r, _Waiter(inst_s.gen), _Waiter(stimulus()), _Waiter(check())

    def testSignal1(self):
        sim = Simulation(self.bench(SignalFunc1, _SignalWaiter))
        sim.run()

    def testSignalTuple1(self):
        sim = Simulation(self.bench(SignalTupleFunc1, _SignalTupleWaiter))
        sim.run()

    def testDelay(self):
        sim = Simulation(self.bench(DelayFunc, _DelayWaiter))
        sim.run()

    def testEdge1(self):
        sim = Simulation(self.bench(EdgeFunc1, _EdgeWaiter))
        sim.run()

    def testEdgeTuple1(self):
        sim = Simulation(self.bench(EdgeTupleFunc1, _EdgeTupleWaiter))
        sim.run()

    def testGeneral(self):
        sim = Simulation(self.bench(GeneralFunc, _Waiter))
        sim.run()



from myhdl import (block, Signal, intbv, instance, always_comb, delay)


@block
def dut():

    rx = Signal(intbv(0, min=-512, max=512))
    a = Signal(intbv(0, min=0, max=256))
    b = Signal(intbv(0, min=0, max=256))
    c = Signal(intbv(0, min=0, max=256))
    d = Signal(intbv(0, min=0, max=256))

    @always_comb
    def comb():
        rx.next = a + b - (c + d)

    @instance
    def check():
        a.next = 0
        b.next = 0
        c.next = 0
        d.next = 0
        for dummy in range(100):
            yield delay(10)
            print(rx)
            a.next = (a + 37) % 256
            b.next = (b + 67) % 256
            c.next = (c + 97) % 256
            d.next = (d + 137) % 256

    return comb, check


def test_bug_39():
    assert dut().verify_convert() == 0




import unittest
from unittest import TestCase

from myhdl import Simulation, Signal, delay, intbv, bin

from bin2gray import bin2gray

MAX_WIDTH = 10

def nextLn(Ln):
    """ Return Gray code Ln+1, given Ln. """
    Ln0 = ['0' + codeword for codeword in Ln]
    Ln1 = ['1' + codeword for codeword in Ln]
    Ln1.reverse()
    return Ln0 + Ln1

class TestOriginalGrayCode(TestCase):

    def testOriginalGrayCode(self):
        
        """ Check that the code is an original Gray code """

        Rn = []
        
        def stimulus(B, G, n):
            for i in range(2**n):
                B.next = intbv(i)
                yield delay(10)
                Rn.append(bin(G, width=n))
        
        Ln = ['0', '1'] # n == 1
        for n in range(2, MAX_WIDTH):
            Ln = nextLn(Ln)
            del Rn[:]
            B = Signal(intbv(1))
            G = Signal(intbv(0))
            dut = bin2gray(B, G, n)
            stim = stimulus(B, G, n)
            sim = Simulation(dut, stim)
            sim.run(quiet=1)
            self.assertEqual(Ln, Rn)


class TestGrayCodeProperties(TestCase):

    def testSingleBitChange(self):
        
        """ Check that only one bit changes in successive codewords """

        
        def test(B, G, G_Z, width):
            B.next = intbv(0)
            yield delay(10)
            for i in range(1, 2**width):
                G_Z.next = G
                B.next = intbv(i)
                yield delay(10)
                diffcode = bin(G ^ G_Z)
                self.assertEqual(diffcode.count('1'), 1)
        
        for width in range(2, MAX_WIDTH):
            B = Signal(intbv(1))
            G = Signal(intbv(0))
            G_Z = Signal(intbv(0))
            dut = bin2gray(B, G, width)
            check = test(B, G, G_Z, width)
            sim = Simulation(dut, check)
            sim.run(quiet=1)


    def testUniqueCodeWords(self):
        
        """ Check that all codewords occur exactly once """

        def test(B, G, width):
            actual = []
            for i in range(2**width):
                B.next = intbv(i)
                yield delay(10)
                actual.append(int(G))
            actual.sort()
            expected = list(range(2**width))
            self.assertEqual(actual, expected)
       
        for width in range(1, MAX_WIDTH):
            B = Signal(intbv(1))
            G = Signal(intbv(0))
            dut = bin2gray(B, G, width)
            check = test(B, G, width)
            sim = Simulation(dut, check)
            sim.run(quiet=1)
            

if __name__ == '__main__':
    unittest.main()


            
            

    

    
        


                

        




from myhdl import (Signal, intbv, instance, delay, ConcatSignal, TristateSignal)
from myhdl._Simulation import Simulation


def bench_SliceSignal():

    s = Signal(intbv(0)[8:])
    a, b, c = s(7), s(5), s(0)
    d, e, f, g = s(8, 5), s(6, 3), s(8, 0), s(4, 3)

    @instance
    def check():
        for i in range(2 ** len(s)):
            s.next = i
            yield delay(10)
            assert s[7] == a
            assert s[5] == b
            assert s[0] == c
            assert s[8:5] == d
            assert s[6:3] == e
            assert s[8:0] == f
            assert s[4:3] == g

    return check


def test_SliceSignal():
    Simulation(bench_SliceSignal()).run()


def bench_SliceSlicedSignal():

    s = Signal(intbv(0)[8:])
    a, b = s(8, 4), s(4, 0)
    aa, ab = a(4, 2), a(2, 0)
    ba, bb = b(4, 2), b(2, 0)

    @instance
    def check():
        for i in range(2 ** len(s)):
            s.next = i
            yield delay(10)
            assert s[8:6] == aa
            assert s[6:4] == ab
            assert s[4:2] == ba
            assert s[2:0] == bb

    return check


def test_SliceSlicedSignal():
    Simulation(bench_SliceSlicedSignal()).run()


def bench_ConcatSignal():

    a = Signal(intbv(0)[5:])
    b = Signal(bool(0))
    c = Signal(intbv(0)[3:])
    d = Signal(intbv(0)[4:])

    s = ConcatSignal(a, b, c, d)

    @instance
    def check():
        for i in range(2 ** len(a)):
            for j in (0, 1):
                for k in range(2 ** len(c)):
                    for m in range(2 ** len(d)):
                        a.next = i
                        b.next = j
                        c.next = k
                        d.next = m
                        yield delay(10)
                        assert s[13:8] == a
                        assert s[7] == b
                        assert s[7:4] == c
                        assert s[4:] == d

    return check


def test_ConcatSignal():
    Simulation(bench_ConcatSignal()).run()


def bench_ConcatSignalWithConsts():

    a = Signal(intbv(0)[5:])
    b = Signal(bool(0))
    c = Signal(intbv(0)[3:])
    d = Signal(intbv(0)[4:])
    e = Signal(intbv(0)[1:])

    c1 = "10"
    c2 = '0'
    c3 = intbv(5)[3:]
    c4 = bool(1)
    c5 = intbv(42)[8:]  # with leading zeroes

    s = ConcatSignal(c1, a, c2, b, c3, c, c4, d, c5, e)

    @instance
    def check():
        for i in range(2 ** len(a)):
            for j in (0, 1):
                for k in range(2 ** len(c)):
                    for m in range(2 ** len(d)):
                        for n in range(2 ** len(e)):
                            a.next = i
                            b.next = j
                            c.next = k
                            d.next = m
                            e.next = n
                            yield delay(10)
                            assert s[29:27] == int(c1, 2)
                            assert s[27:22] == a
                            assert s[21] == int(c2, 2)
                            assert s[20] == b
                            assert s[20:17] == c3
                            assert s[17:14] == c
                            assert s[13] == c4
                            assert s[13:9] == d
                            assert s[9:1] == c5
                            assert s[1:] == e

    return check


def test_ConcatSignalWithConsts():
    Simulation(bench_ConcatSignalWithConsts()).run()


def bench_ConcatSignalWithNegs():

    Am = 2 ** (5 - 1)
    Cm = 2 ** (3 - 1)
    Dm = 2 ** (4 - 1)

    a = Signal(intbv(-1, min=-Am, max=Am))
    b = Signal(bool(0))
    c = Signal(intbv(-1, min=-Cm, max=Cm))
    d = Signal(intbv(-1, min=-Dm, max=Dm))

    s = ConcatSignal(a, b, c, d)

    @instance
    def check():
        for i in range(-Am, Am):
            for j in (0, 1):
                for k in range(-Cm, Cm):
                    for m in range(-Dm, Dm):
                        a.next = i
                        b.next = j
                        c.next = k
                        d.next = m
                        yield delay(10)

                        assert s[13:8] == a[len(a):]
                        assert s[7] == b
                        assert s[7:4] == c[len(c):]
                        assert s[4:] == d[len(d):]

    return check


def test_ConcatSignalWithNegs():
    Simulation(bench_ConcatSignalWithNegs()).run()


def bench_ConcatConcatedSignal():

    aa = Signal(intbv(0)[2:0])
    ab = Signal(intbv(0)[2:0])
    a = ConcatSignal(aa, ab)

    ba = Signal(intbv(0)[2:0])
    bb = Signal(intbv(0)[2:0])
    b = ConcatSignal(ba, bb)

    s = ConcatSignal(a, b)

    @instance
    def check():
        for iaa in range(2 ** len(aa)):
            for iab in range(2 ** len(ab)):
                for iba in range(2 ** len(ba)):
                    for ibb in range(2 ** len(bb)):
                        aa.next = iaa
                        ab.next = iab
                        ba.next = iba
                        bb.next = ibb
                        yield delay(10)
                        assert s[8:6] == aa
                        assert s[6:4] == ab
                        assert s[4:2] == ba
                        assert s[2:0] == bb

    return check


def test_ConcatConcatedSignal():
    Simulation(bench_ConcatConcatedSignal()).run()


def bench_TristateSignal():
    s = TristateSignal(intbv(0)[8:])
    a = s.driver()
    b = s.driver()
    c = s.driver()

    @instance
    def check():
        assert s == None
        a.next = 1
        yield delay(10)
        assert s == a
        a.next = None
        b.next = 122
        yield delay(10)
        assert s == b
        b.next = None
        c.next = 233
        yield delay(10)
        assert s == c
        c.next = None
        yield delay(10)
        assert s == None

    return check


def test_TristateSignal():
    Simulation(bench_TristateSignal()).run()



from myhdl import (block, Signal, intbv, always)


@block
def module42_2(sigin, sigout):

    # Using @always(sigin) only warns, but using @always_comb breaks.
    # The reason is that len(sigout) is interpreted as sigout being used as
    # an input.
    @always(sigin)
    def output():
        sigout.next = sigin[len(sigout):]

    return output


sigin = Signal(intbv(0)[2:])
sigout = Signal(intbv(0)[2:])


def test_bug_42_2():
    module42_2(sigin, sigout).convert(hdl='VHDL')


module42_2(sigin, sigout).convert(hdl='VHDL')




import myhdl
from myhdl import *

from timer import timer_sig, timer_var

def test_timer_array(timer):

    MAXVAL = 1234

    clock = Signal(bool())
    reset = Signal(bool())
    flag0 = Signal(bool())
    flag1 = Signal(bool())
    flag2 = Signal(bool())
    flag3= Signal(bool())
    flag4 = Signal(bool())
    flag5 = Signal(bool())
    flag6 = Signal(bool())
    flag7 = Signal(bool())

    dut = [None] * 8

    dut[0] = timer(flag0, clock, reset, MAXVAL)
    dut[1] = timer(flag1, clock, reset, MAXVAL)
    dut[2] = timer(flag2, clock, reset, MAXVAL)
    dut[3] = timer(flag3, clock, reset, MAXVAL)
    dut[4] = timer(flag4, clock, reset, MAXVAL)
    dut[5] = timer(flag5, clock, reset, MAXVAL)
    dut[6] = timer(flag6, clock, reset, MAXVAL)
    dut[7] = timer(flag7, clock, reset, MAXVAL)

    @instance
    def clkgen():
        clock.next = 0
        reset.next = 0
        yield delay(10)
        reset.next = 1
        yield delay(10)
        reset.next = 0
        yield delay(10)
        for i in range(2**24):
            clock.next = not clock
            yield delay(10)

    @instance
    def monitor():
        count = intbv(0, min=0, max=MAXVAL+1)
        seen = False
        while True:
            yield clock.posedge
            if seen:
                if flag0:
                    assert count == MAXVAL
                else:
                    count += 1
            if flag0 or flag1 or flag2 or flag3 or flag4 or flag5 or flag6 or flag7:
                seen = True
                count[:] = 0

    return dut, clkgen, monitor

if __name__ == '__main__':
    sim = Simulation(test_timer_array(timer_var))
    sim.run()
        



import os
path = os.path
from random import randrange

from myhdl import (block, Signal, intbv, delay, concat, instance, downrange,)
# from .util import setupCosimulation  # not found!

COSET = 0x55


def calculateHecRef(header):
    """ Return hec for an ATM header.

    Reference version.
    The hec polynomial is 1 + x + x**2 + x**8.
    """
    hec = intbv(0)
    for bit in header[32:]:
        hec[8:] = concat(hec[7:2],
                         bit ^ hec[1] ^ hec[7],
                         bit ^ hec[0] ^ hec[7],
                         bit ^ hec[7]
                        )
    return hec ^ COSET


def calculateHecFunc(header):
    """ Return hec for an ATM header.

    Translatable version.
    The hec polynomial is 1 + x + x**2 + x**8.
    """
    h = intbv(0)[8:]
    for i in downrange(len(header)):
        bit = header[i]
        h[:] = concat(h[7:2],
                      bit ^ h[1] ^ h[7],
                      bit ^ h[0] ^ h[7],
                      bit ^ h[7]
                      )
    h ^= COSET
    return h


def calculateHecTask(hec, header):
    """ Calculate hec for an ATM header.

    Translatable version.
    The hec polynomial is 1 + x + x**2 + x**8.
    """
    h = intbv(0)[8:]
    for i in downrange(len(header)):
        bit = header[i]
        h[:] = concat(h[7:2],
                      bit ^ h[1] ^ h[7],
                      bit ^ h[0] ^ h[7],
                      bit ^ h[7]
                      )
    h ^= COSET
    hec[:] = h


@block
def HecCalculatorPlain(hec, header):
    """ Hec calculation module.

    Plain version.
    """

    @instance
    def comb():
        h = intbv(0)[8:]
        while 1:
            yield header
            h[:] = 0
            for i in downrange(len(header)):
                bit = header[i]
                h[:] = concat(h[7:2],
                              bit ^ h[1] ^ h[7],
                              bit ^ h[0] ^ h[7],
                              bit ^ h[7]
                              )
            hec.next = h ^ COSET

    return comb


def HecCalculatorFunc(hec, header):
    """ Hec calculation module.

    Version with function call.
    """
    while 1:
        yield header
        hec.next = calculateHecFunc(header=header)


def HecCalculatorTask(hec, header):
    """ Hec calculation module.

    Version with task call.
    """
    h = intbv(0)[8:]
    while 1:
        yield header
        calculateHecTask(h, header)
        hec.next = h


def HecCalculatorTask2(hec, header):
    """ Hec calculation module.

    Version with task call.
    """
    h = intbv(0)[8:]
    while 1:
        yield header
        calculateHecTask(header=header, hec=h)
        hec.next = h

#
# def HecCalculator_v(name, hec, header):
#     return setupCosimulation(**locals())


headers = [ 0x00000000,
            0x01234567,
            0xbac6f4ca
          ]

headers.extend([randrange(2 ** 32 - 1) for i in range(10)])
headers = tuple(headers)


@block
def HecBench(HecCalculator):

    hec = Signal(intbv(0)[8:])
    header = Signal(intbv(-1)[32:])

    heccalc_inst = HecCalculator(hec, header)

    @instance
    def stimulus():
        for i in range(len(headers)):
            header.next = headers[i]
            yield delay(10)
            print(hec)

    return stimulus, heccalc_inst

# # def testPlain(self):
# #     sim = self.bench(HecCalculatorPlain)
# #     Simulation(sim).run()

# # def testFunc(self):
# #     sim = self.bench(HecCalculatorFunc)
# #     Simulation(sim).run()

# # def testTask(self):
# #     sim = self.bench(HecCalculatorTask)
# #     Simulation(sim).run()

# # def testTask2(self):
# #     sim = self.bench(HecCalculatorTask2)
# #     Simulation(sim).run()


def testPlain():
    assert HecBench(HecCalculatorPlain).verify_convert() == 0



from myhdl import (block, Signal, intbv, always, always_comb, toVHDL)


@block
def issue_18(dout, din, addr, we, clk, depth=128):
    """  Ram model """

    mem = [Signal(intbv(0)[8:]) for __ in range(depth)]

    @always(clk.posedge)
    def write():
        if we:
            mem[addr].next = din

    @always_comb
    def read():
        dout.next = mem[addr]

    return write, read


dout = Signal(intbv(0)[8:])
dout_v = Signal(intbv(0)[8:])
din = Signal(intbv(0)[8:])
addr = Signal(intbv(0)[7:])
we = Signal(bool(0))
clk = Signal(bool(0))


def test_issue_18():
    toVHDL.std_logic_ports = True
    assert issue_18(dout, din, addr, we, clk).analyze_convert() == 0






from myhdl import (block, Signal, ResetSignal, modbv, always_seq, concat,
                   instance, delay, StopSimulation)
from myhdl.conversion import analyze, verify

"""
This set of tests exercises a peculiar scenario where an expanded
interface Signal is flagged as having multiple drivers.  This appears
to be a name collision in the name expansion and was introduced in
08519b4.
"""


class Intf1(object):

    def __init__(self):
        self.sig1 = Signal(bool(0))
        self.sig2 = Signal(bool(0))
        self.sig3 = Signal(modbv(0)[8:])


class Intf2(object):

    def __init__(self):
        self.sig1 = Signal(bool(0))
        self.sig2 = Signal(bool(0))
        self.sig3 = Signal(modbv(0)[8:])
        self.intf = Intf1()


@block
def use_nested_intf(clock, reset, intf1, intf2):

    sig1 = Signal(bool(0))
    sig2 = Signal(bool(0))

    @always_seq(clock.posedge, reset)
    def proc():
        if intf1.sig1:
            sig1.next = True
            sig2.next = False
        else:
            sig1.next = False
            sig2.next = True

        intf2.sig1.next = sig1
        intf2.sig2.next = sig2 or intf1.sig2
        intf2.sig3.next = ~intf1.sig3
        intf2.intf.sig1.next = intf2.sig2
        intf2.intf.sig2.next = intf2.intf.sig1

    return proc


@block
def something_peculiar(clock, reset, intf1, intf2):

    @always_seq(clock.posedge, reset)
    def proc():
        # remove the if/else and leave just the line in the
        # if clause the error does not occur, inlcude the if/else
        # and the error occurs
        if intf1.sig3 > 0:  # remove no error
            intf2.sig1.next = not intf1.sig1
            intf2.sig2.next = not intf1.sig2
            intf2.sig3.next = intf1.sig3 + intf2.sig3
        else:  # remove no error
            intf2.sig3.next = 0  # remove no error

    return proc


@block
def interfaces_top(clock, reset, sdi, sdo, nested):

    intf1, intf2, intf3 = Intf1(), Intf2(), Intf1()

    inst1 = use_nested_intf(clock, reset, intf1, intf2)
    inst2 = something_peculiar(clock, reset, intf2, intf3)

    @always_seq(clock.posedge, reset)
    def assigns():
        intf1.sig1.next = sdi
        intf1.sig2.next = not sdi
        intf1.sig3.next = concat(intf1.sig3[7:1], sdi)
        sdo.next = intf3.sig1 | intf3.sig2 | intf3.sig3[2]
        nested.next = intf2.intf.sig2

    return inst1, inst2, assigns


@block
def c_testbench_one():
    """ yet another interface test.
    This test is used to expose a particular bug that was discovered
    during the development of interface conversion.  The structure
    used in this example caused and invalid multiple driver error.
    """
    clock = Signal(bool(0))
    reset = ResetSignal(0, active=1, isasync=False)
    sdi = Signal(bool(0))
    sdo = Signal(bool(0))
    nested = Signal(bool())
    tbdut = interfaces_top(clock, reset, sdi, sdo, nested)

    @instance
    def tbclk():
        clock.next = False
        while True:
            yield delay(3)
            clock.next = not clock

    # there is an issue when using bools with variables and
    # VHDL conversion, this might be an expected limitation?
    # expected = (False, False, False, True, True, True,
    #             False, True, False, True)
    # use a tuple-of-ints instead of the above tuple-of-bools
    expected = (0, 0, 0, 1, 1, 1, 0, 1, 0, 1)
    ra = reset.active

    @instance
    def tbstim():
        sdi.next = False
        reset.next = ra
        yield delay(13)
        reset.next = not ra
        yield clock.posedge
        for ii in range(10):
            print("sdi: %d, sdo: %d, nested: %d" % (sdi, sdo, nested))
            expected_bit = expected[ii]
            assert sdo == expected_bit
            sdi.next = not sdi
            yield clock.posedge

        raise StopSimulation

    return tbclk, tbstim, tbdut


def test_one_testbench():
    inst = c_testbench_one()
    inst.run_sim()


def test_one_analyze():
    clock = Signal(bool(0))
    reset = ResetSignal(0, active=1, isasync=False)
    sdi = Signal(bool(0))
    sdo = Signal(bool(0))
    nested = Signal(bool(0))
    assert analyze(interfaces_top(clock, reset, sdi, sdo, nested)) == 0


def test_one_verify():
    assert verify(c_testbench_one()) == 0


def test_conversion():
    inst = c_testbench_one()
    inst.convert(hdl='Verilog')
    inst.convert(hdl='VHDL')



from math import atan, sqrt, ceil, floor, pi

import myhdl
from myhdl import *

t_State = enum("WAITING", "CALCULATING")

def SineComputer(cos_z0, sin_z0, done, z0, start, clock, reset):
    
    """ Sine and cosine computer.

    This module computes the sine and cosine of an input angle. The
    floating point numbers are represented as integers by scaling them
    up with a factor corresponding to the number of bits after the point.

    Ports:
    -----
    cos_z0: cosine of the input angle
    sin_z0: sine of the input angle
    done: output flag indicating completion of the computation
    z0: input angle; -pi/2 <= z0 <= pi/2
    start: input that starts the computation on a posedge
    clock: clock input
    reset: reset input

    """

    # angle input bit width
    W = len(z0)

    # angle input z0 represents number between -pi/2 and pi/2
    # scaling factor corresponds to the nr of bits after the point
    M = 2 ** (W-2)

    # nr of iterations equals nr of significant input bits
    N = W-1
    
    # calculate X0
    An = 1.0
    for i in range(N):
        An *= (sqrt(1 + 2**(-2*i)))
        
    # X0
    X0 = int(round(M*1/An))
    
    # tuple with elementary angles
    angles = tuple([int(round(M*atan(2**(-i)))) for i in range(N)])
    
    # iterative cordic processor
    @instance
    def processor():
        
        x = intbv(0, min=sin_z0.min, max=sin_z0.max)
        y = intbv(0, min=sin_z0.min, max=sin_z0.max)
        z = intbv(0, min=z0.min, max=z0.max)
        dx = intbv(0, min=sin_z0.min, max=sin_z0.max)
        dy = intbv(0, min=sin_z0.min, max=sin_z0.max)
        dz = intbv(0, min=z0.min, max=z0.max)
        i = intbv(0, min=0, max=N)
        state = t_State.WAITING

        while True:
            yield clock.posedge, reset.posedge

            if reset:
                state = t_State.WAITING
                cos_z0.next = 1
                sin_z0.next = 0
                done.next = False
                x[:] = 0
                y[:] = 0
                z[:] = 0
                i[:] = 0

            else:
                if state == t_State.WAITING:
                    if start:
                        x[:] = X0
                        y[:] = 0
                        z[:] = z0
                        i[:] = 0
                        done.next = False
                        state = t_State.CALCULATING

                elif state == t_State.CALCULATING:
                    dx[:] = y >> i
                    dy[:] = x >> i
                    dz[:] = angles[int(i)]
                    if (z >= 0):
                        x -= dx
                        y += dy
                        z -= dz
                    else:
                        x += dx
                        y -= dy
                        z += dz
                    if i == N-1:
                        cos_z0.next = x
                        sin_z0.next = y
                        state = t_State.WAITING
                        done.next = True
                    else:
                        i += 1

    return processor

def SineComputer_v(cos_z0, sin_z0, done, z0, start, clock, reset):
    toVerilog(SineComputer, cos_z0, sin_z0, done, z0, start, clock, reset)
    conversion.analyze(SineComputer, cos_z0, sin_z0, done, z0, start, clock, reset)
    cmd = "cver -q +loadvpi=./myhdl_vpi:vpi_compat_bootstrap " + \
          "SineComputer.v tb_SineComputer.v"
    return Cosimulation(cmd, **locals())
##     cmd = "iverilog SineComputer.v tb_SineComputer.v"
##     import os
##     os.system(cmd)
##     return Cosimulation("vvp -m ./myhdl.vpi a.out", **locals())

       




from myhdl import (block, Signal, intbv, always_seq, ResetSignal, always)


@block
def gray_counter_bug_boolop (clk, reset, enable, gray_count):

    q = Signal(intbv(0)[10:])
    no_ones_below = Signal(intbv(0)[10:])
    q_msb = Signal(bool(0))

    @always_seq(clk.posedge, reset=reset)
    def seq():
        if enable:
            q.next[0] = not q[0]
            for i in range(1, 9):
                q.next[i] = q[i] ^ (q[i - 1] and no_ones_below[i - 1])
            q.next[8] = q[8] ^ (q_msb and no_ones_below[7])

    @always(q, no_ones_below)
    def comb():
        q_msb.next = q[8] or q[7]
        no_ones_below.next[0] = 1
        for j in range(1, 10):
            no_ones_below.next[j] = no_ones_below[j - 1] and not q[j - 1]
        gray_count.next[8:] = q[9:1]

    return comb, seq


clk = Signal(bool(0))
reset = ResetSignal(0, active=0, isasync=True)
enable = Signal(bool(0))
gray_count = Signal(intbv(0)[8:])


def test_bug_boolop():
    try:
        gray_counter_bug_boolop(clk, reset, enable, gray_count).convert(hdl='Verilog')
        gray_counter_bug_boolop(clk, reset, enable, gray_count).convert(hdl='VHDL')
    except:
        assert False
